# Copyright (c) 2008 Joost Cassee
# Licensed under the terms of the MIT License (see LICENSE.txt)

from django.conf import settings

PREFIX_DEFAULT_LOCALE = getattr(settings, "PREFIX_DEFAULT_LOCALE", True)

REDIRECT_LOCALE_INDEPENDENT_PATHS = getattr(settings,
        'REDIRECT_LOCALE_INDEPENDENT_PATHS', False)

LOCALE_INDEPENDENT_PATHS = getattr(settings, 'LOCALE_INDEPENDENT_PATHS', ())



# Copyright (c) 2008 Joost Cassee
# Licensed under the terms of the MIT License (see LICENSE.txt)

import re
from django import http
from django.conf import settings
from django.core import urlresolvers
from django.http import HttpResponseRedirect
from django.utils import translation
import localeurl
from localeurl.utils import strip_locale_prefix, is_locale_independent, get_language

# Make sure the default language is in the list of supported languages
assert get_language(settings.LANGUAGE_CODE) is not None, \
        "Please ensure that settings.LANGUAGE_CODE is in settings.LANGUAGE."

class LocaleURLMiddleware(object):
    """
    Middleware that sets the language based on the request path prefix and
    strips that prefix from the path. It will also automatically redirect any
    path without a prefix, unless PREFIX_DEFAULT_LOCALE is set to True.
    Exceptions are paths beginning with MEDIA_URL or matching any regular
    expression from LOCALE_INDEPENDENT_PATHS from the project settings.

    For example, the path '/en/admin/' will set request.LANGUAGE_CODE to 'en'
    and request.path to '/admin/'.

    If you use this middleware the django.core.urlresolvers.reverse function
    is be patched to return paths with locale prefix.
    """
    def strip_locale_from_request(self,request):
        check = re.search(r'^/([^/]+)(/.*)$', request.path_info)
        if check is not None:
            locale = check.group(1)
            if get_language(locale) == locale:
                request.path_info = check.group(2)
                return locale
        return None

    def process_request(self, request):
        locale = self.strip_locale_from_request(request)
        if locale is not None:
            if (is_locale_independent(request.path_info)
                    and localeurl.REDIRECT_LOCALE_INDEPENDENT_PATHS):
                return HttpResponseRedirect(request.path_info)
            if (locale == get_language(settings.LANGUAGE_CODE)
                    and not localeurl.PREFIX_DEFAULT_LOCALE):
                return HttpResponseRedirect(request.path_info)
            translation.activate(locale)
            request.LANGUAGE_CODE = translation.get_language()
        elif (not is_locale_independent(request.path)
                and localeurl.PREFIX_DEFAULT_LOCALE):
            return redirect_locale(request)

    def process_response(self, request, response):
        if 'Content-Language' not in response:
            response['Content-Language'] = translation.get_language()
        translation.deactivate()
        return response

def redirect_locale(request, path=None, locale=None):
    """
    Prepend a locale to the path. If path is omitted, the request path is used.
    If locale is omitted the current locale is used, or the default from
    settings if the request does not contain LANGUAGE_CODE.
    """
    if path is None:
        path = request.get_full_path()
    path = strip_locale_prefix(path)
    if locale is None:
        try:
            locale = get_language(request.LANGUAGE_CODE)
        except AttributeError:
            locale = get_language(settings.LANGUAGE_CODE)
    return HttpResponseRedirect('/' + locale + path)

# Replace reverse function
def reverse(viewname, urlconf=None, args=None, kwargs=None):
    """
    Returns the URL from a view name, taking into account locale prefixes.
    """
    path = django_reverse(viewname, urlconf, args, kwargs)
    if is_locale_independent(path):
        return path
    lang = get_language(translation.get_language())
    def_lang = get_language(settings.LANGUAGE_CODE)
    if lang == def_lang and not localeurl.PREFIX_DEFAULT_LOCALE:
        return path
    return '/' + translation.get_language() + path
django_reverse = urlresolvers.reverse
urlresolvers.reverse = reverse



# Copyright (c) 2008 Joost Cassee
# Licensed under the terms of the MIT License (see LICENSE.txt)

from django import template
from django.template import Node, Token, TemplateSyntaxError
from django.template import resolve_variable, defaulttags
from django.template.defaultfilters import stringfilter
from django.conf import settings
from django.utils import translation
import localeurl
from localeurl.utils import strip_locale_prefix, get_language

register = template.Library()


def chlocale(path, locale):
    """Changes the path's locale prefix."""
    if get_language(locale) == get_language(settings.LANGUAGE_CODE) \
            and not localeurl.PREFIX_DEFAULT_LOCALE:
        return rmlocale(path)
    else:
        return '/' + get_language(locale) + rmlocale(path)

chlocale = stringfilter(chlocale)
register.filter('chlocale', chlocale)


def rmlocale(url):
    """Removes the locale prefix from the path."""
    return strip_locale_prefix(url)

rmlocale = stringfilter(rmlocale)
register.filter('rmlocale', rmlocale)


def locale_url(parser, token):
    """
    Renders the url for the view with another locale prefix. The syntax is
    like the 'url' tag, only with a locale before the view.

    Examples:
      {% locale_url "de" cal.views.day day %}
      {% locale_url "nl" cal.views.home %}
      {% locale_url "en-gb" cal.views.month month as month_url %}
    """
    bits = token.split_contents()
    if len(bits) < 3:
        raise TemplateSyntaxError("'%s' takes at least two arguments:"
                " the locale and a view" % bits[0])
    urltoken = Token(token.token_type, bits[0] + ' ' + ' '.join(bits[2:]))
    urlnode = defaulttags.url(parser, urltoken)
    return LocaleURLNode(bits[1], urlnode)

class LocaleURLNode(Node):
    def __init__(self, locale, urlnode):
        self.locale = locale
        self.urlnode = urlnode

    def render(self, context):
        locale = resolve_variable(self.locale, context)
        path = self.urlnode.render(context)
        if self.urlnode.asvar:
            self.urlnode.render(context)
            context[self.urlnode.asvar] = chlocale(context[self.urlnode.asvar],
                    locale)
            return ''
        else:
            return chlocale(path, locale)

register.tag('locale_url', locale_url)



# Copyright (c) 2008 Joost Cassee
# Licensed under the terms of the MIT License (see LICENSE.txt)

from django.conf import settings
from localeurl import LOCALE_INDEPENDENT_PATHS

SUPPORTED_LOCALES = dict(settings.LANGUAGES)

def is_locale_independent(path):
    """
    Returns whether the path is locale-independent.

    A path is independent if it starts with MEDIA_URL or it is matched by any
    pattern from LOCALE_INDEPENDENT_PATHS.
    """
    if settings.MEDIA_URL and path.startswith(settings.MEDIA_URL):
        return True
    for path_re in LOCALE_INDEPENDENT_PATHS:
        if path_re.search(path):
            return True
    return False

def strip_locale_prefix(path):
    """
    Returns the path without the locale prefix. If the path does not begin
    with a locale it is returned without change.
    """
    for lang in settings.LANGUAGES:
        locale = '/' + lang[0] + '/'
        if path.startswith(locale):
            return path[len(locale)-1:]
    return path

def get_language(locale):
    """
    Returns the supported language (from settings.LANGUAGES)
    """
    if locale in SUPPORTED_LOCALES:
        return locale
    elif locale[:2] in SUPPORTED_LOCALES:
        return locale[:2]
    else:
        return None



#!/usr/bin/env python
#
# Copyright (c) 2008 Joost Cassee
# Licensed under the terms of the MIT License (see LICENSE.txt)

from distutils.core import setup
from distutils.filelist import findall
import metadata

app_name = metadata.name
version = metadata.version

setup(
    name = "django-%s" % app_name,
    version = version,

    packages = [app_name, '%s.templatetags' % app_name],

    author = "Joost Cassee",
    author_email = "joost@cassee.net",
    description = "A Django application that allow you to specify the" \
            " language of a page in the URL.",
    long_description = \
"""
Suppose you have a Django website in multiple languages. The localeurl
application allow you to specify the language of a page in the URL, like so:
`http://www.example.com/nl/company/profile`. Any URLs without language prefix
will be redirected to add the prefix for the default language.

Some reasons for using localeurl:

* Search engines will index all languages.
* Every page should have a unique URL. If you feel that different languages
  means different pages, then each language should get its own unique URL.
* Setting the language for the website should be done using a POST request
  (because it influences subsequent page views, see `Django ticket #3651`_).
  You might not want to use POST requests.

.. _Django ticket #3651: http://code.djangoproject.com/ticket/3651
""",
    license = "MIT License",
    keywords = "django i18n",
    classifiers = [
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Localization',
    ],
    platforms = ['any'],
    url = "http://code.google.com/p/django-%s/" % app_name,
    download_url = "http://code.google.com/p/django-%s/downloads/list" \
            % app_name,
)
